﻿// download latest version from http://www.console-dev.de
using UnityEngine;
using UnityEditor;
using System.Collections.Generic;

class MakeMesh
{
    [MenuItem("Example/Make Mesh")]
    static void DoMenuItem()
    {
        const string path = "Assets/MyMesh.asset";

        Mesh mesh = null;
        if(System.IO.File.Exists(path))
            mesh = AssetDatabase.LoadAssetAtPath<Mesh>(path);
        else
        {
            mesh = new Mesh();
            AssetDatabase.CreateAsset(mesh, path);
        }

        mesh.Clear();

        var vertices = new List<Vector3>();
        var triangles = new List<int>();
        var triangleIndex = 0;
        var uvs = new List<Vector2>();
        var localPos = new List<Vector2>();
        var frameIndices = new List<Vector2>();
        var colors = new List<Color>();

        // hard coded seed so the mesh comes out the same always
        Random.InitState(123456);

        for(var x = -49; x < 50; ++x)
        {
            for(var z = -49; z < 50; ++z)
            {
                var s = new Vector3(x, Random.Range(-10, 10+1), z)*1.5f;

                // 4 vertices representing the shape of a quad
                var v0 = new Vector3(-0.5f, -0.5f, 0);
                var v1 = new Vector3(-0.5f, 0.5f, 0);
                var v2 = new Vector3(0.5f, 0.5f, 0);
                var v3 = new Vector3(0.5f, -0.5f, 0);

                vertices.Add(v0 + s);
                vertices.Add(v1 + s);
                vertices.Add(v2 + s);
                vertices.Add(v3 + s);

                // add local position to mesh to make it easier in the shader
                localPos.Add(new Vector2(v0.x, v0.y));
                localPos.Add(new Vector2(v1.x, v1.y));
                localPos.Add(new Vector2(v2.x, v2.y));
                localPos.Add(new Vector2(v3.x, v3.y));

                // add tex-coords
                var uv0 = new Vector2(0, 1);
                var uv1 = new Vector2(0, 0);
                var uv2 = new Vector2(1, 0);
                var uv3 = new Vector2(1, 1);

                uvs.Add(uv0);
                uvs.Add(uv1);
                uvs.Add(uv2);
                uvs.Add(uv3);

                // add random color for each billboard
                Color color = Color.Lerp(Color.green, Color.yellow, Random.Range(0.0f, 1));
                color = Color.Lerp(color, new Color(0.5f,0.5f,1), Random.Range(0.0f, 1));
                colors.Add(color);
                colors.Add(color);
                colors.Add(color);
                colors.Add(color);

                // add random starting frame index for each billboard
                // 8*8 assumes the texture contains 8 columns and 8 rows
                var frameIndex = new Vector2(Random.Range(0, 8 * 8), 0);
                frameIndices.Add(frameIndex);
                frameIndices.Add(frameIndex);
                frameIndices.Add(frameIndex);
                frameIndices.Add(frameIndex);

                // left triangle of quad
                triangles.Add(triangleIndex+0);
                triangles.Add(triangleIndex+1);
                triangles.Add(triangleIndex+2);

                // right triangle of quad
                triangles.Add(triangleIndex+2);
                triangles.Add(triangleIndex+3);
                triangles.Add(triangleIndex+0);

                triangleIndex += 4;
            }
        }

        mesh.vertices = vertices.ToArray();
        mesh.uv = uvs.ToArray();
        mesh.triangles = triangles.ToArray();
        mesh.uv2 = localPos.ToArray();
        mesh.uv3 = frameIndices.ToArray();
        mesh.colors = colors.ToArray();

        mesh.RecalculateNormals();
        mesh.RecalculateBounds();
        EditorUtility.SetDirty(mesh);
    }
}
